@@ -7,6 +7,7 @@ import os
 import platform
 import socket
 import sys
+import tempfile
 import threading
 import time
 from contextlib import suppress
@@ -14,14 +15,7 @@ from pathlib import Path
 from socket import AddressFamily
 from ssl import SSLContext, SSLError
 from threading import Thread
-from typing import (
-    Any,
-    Iterable,
-    Iterator,
-    NoReturn,
-    TypeVar,
-    cast,
-)
+from typing import Any, Generator, Iterable, Iterator, NoReturn, TypeVar, cast
 
 import psutil
 import pytest
@@ -34,21 +28,22 @@ from anyio import (
     BrokenResourceError,
     BusyResourceError,
     ClosedResourceError,
+    EndOfStream,
     Event,
-    ExceptionGroup,
     TypedAttributeLookupError,
     connect_tcp,
     connect_unix,
     create_connected_udp_socket,
+    create_connected_unix_datagram_socket,
     create_task_group,
     create_tcp_listener,
     create_udp_socket,
+    create_unix_datagram_socket,
     create_unix_listener,
     fail_after,
     getaddrinfo,
     getnameinfo,
     move_on_after,
-    sleep,
     wait_all_tasks_blocked,
 )
 from anyio.abc import (
@@ -58,12 +53,13 @@ from anyio.abc import (
     SocketListener,
     SocketStream,
 )
+from anyio.lowlevel import checkpoint
 from anyio.streams.stapled import MultiListener
 
-if sys.version_info >= (3, 8):
-    from typing import Literal
-else:
-    from typing_extensions import Literal
+if sys.version_info < (3, 11):
+    from exceptiongroup import ExceptionGroup
+
+from typing import Literal
 
 AnyIPAddressFamily = Literal[
     AddressFamily.AF_UNSPEC, AddressFamily.AF_INET, AddressFamily.AF_INET6
@@ -87,6 +83,10 @@ if socket.has_ipv6:
         has_ipv6 = True
 
 skip_ipv6_mark = pytest.mark.skipif(not has_ipv6, reason="IPv6 is not available")
+skip_unix_abstract_mark = pytest.mark.skipif(
+    not sys.platform.startswith("linux"),
+    reason="Abstract namespace sockets is a Linux only feature",
+)
 
 
 @pytest.fixture
@@ -206,7 +206,9 @@ class TestTCPStream:
             thread.start()
             response = b""
             while len(response) < len(buffer):
-                response += await stream.receive()
+                chunk = await stream.receive()
+                assert isinstance(chunk, bytes)
+                response += chunk
 
         thread.join()
         assert response == buffer
@@ -485,7 +487,8 @@ class TestTCPStream:
         self, family: AnyIPAddressFamily, caplog: LogCaptureFixture
     ) -> None:
         """
-        Tests that there won't be any leftover Futures that don't get their exceptions retrieved.
+        Test that there won't be any leftover Futures that don't get their exceptions
+        retrieved.
 
         See https://github.com/encode/httpcore/issues/382 for details.
 
@@ -494,28 +497,30 @@ class TestTCPStream:
         def serve() -> None:
             sock, addr = server_sock.accept()
             event.wait(3)
+            sock.close()
             del sock
             gc.collect()
 
-        server_sock = socket.socket(family, socket.SOCK_STREAM)
-        server_sock.settimeout(1)
-        server_sock.bind(("localhost", 0))
-        server_sock.listen()
-        server_addr = server_sock.getsockname()[:2]
-        event = threading.Event()
-        thread = Thread(target=serve)
-        thread.start()
-        async with await connect_tcp(*server_addr) as stream:
-            await stream.send(b"GET")
-            event.set()
-            with pytest.raises(BrokenResourceError):
-                await stream.receive()
+        with socket.socket(family, socket.SOCK_STREAM) as server_sock:
+            server_sock.settimeout(1)
+            server_sock.bind(("localhost", 0))
+            server_sock.listen()
+            server_addr = server_sock.getsockname()[:2]
+            event = threading.Event()
+            thread = Thread(target=serve)
+            thread.start()
+            async with await connect_tcp(*server_addr) as stream:
+                await stream.send(b"GET")
+                event.set()
+                with pytest.raises(BrokenResourceError):
+                    await stream.receive()
 
-        thread.join()
-        gc.collect()
-        assert not caplog.text
+            thread.join()
+            gc.collect()
+            assert not caplog.text
 
 
+@pytest.mark.network
 class TestTCPListener:
     async def test_extra_attributes(self, family: AnyIPAddressFamily) -> None:
         async with await create_tcp_listener(
@@ -645,13 +650,19 @@ class TestTCPListener:
 
     async def test_close_from_other_task(self, family: AnyIPAddressFamily) -> None:
         listener = await create_tcp_listener(local_host="localhost", family=family)
-        with pytest.raises(ClosedResourceError):
+        with pytest.raises(ExceptionGroup) as exc:
             async with create_task_group() as tg:
                 tg.start_soon(listener.serve, lambda stream: None)
                 await wait_all_tasks_blocked()
                 await listener.aclose()
                 tg.cancel_scope.cancel()
 
+        assert len(exc.value.exceptions) == 1
+        assert isinstance(exc.value.exceptions[0], ExceptionGroup)
+        nested_grp = exc.value.exceptions[0]
+        assert len(nested_grp.exceptions) == 1
+        assert isinstance(nested_grp.exceptions[0], ExceptionGroup)
+
     async def test_send_after_eof(self, family: AnyIPAddressFamily) -> None:
         async def handle(stream: SocketStream) -> None:
             async with stream:
@@ -671,13 +682,36 @@ class TestTCPListener:
                         try:
                             message = client.recv(10)
                         except BlockingIOError:
-                            await sleep(0)
+                            await checkpoint()
                         else:
                             assert message == b"Hello\n"
                             break
 
             tg.cancel_scope.cancel()
 
+    async def test_eof_after_send(self, family: AnyIPAddressFamily) -> None:
+        """Regression test for #701."""
+        received_bytes = b""
+
+        async def handle(stream: SocketStream) -> None:
+            nonlocal received_bytes
+            async with stream:
+                received_bytes = await stream.receive()
+                with pytest.raises(EndOfStream), fail_after(1):
+                    await stream.receive()
+
+            tg.cancel_scope.cancel()
+
+        multi = await create_tcp_listener(family=family, local_host="localhost")
+        async with multi, create_task_group() as tg:
+            with socket.socket(family) as client:
+                client.connect(multi.extra(SocketAttribute.local_address))
+                client.send(b"Hello")
+                client.shutdown(socket.SHUT_WR)
+                await multi.serve(handle)
+
+        assert received_bytes == b"Hello"
+
     @skip_ipv6_mark
     @pytest.mark.skipif(
         sys.platform == "win32",
@@ -705,9 +739,20 @@ class TestTCPListener:
     sys.platform == "win32", reason="UNIX sockets are not available on Windows"
 )
 class TestUNIXStream:
-    @pytest.fixture
-    def socket_path(self, tmp_path_factory: TempPathFactory) -> Path:
-        return tmp_path_factory.mktemp("unix").joinpath("socket")
+    @pytest.fixture(
+        params=[
+            "path",
+            pytest.param("abstract", marks=[skip_unix_abstract_mark]),
+        ]
+    )
+    def socket_path(self, request: SubRequest) -> Generator[Path, None, None]:
+        # Use stdlib tempdir generation
+        # Fixes `OSError: AF_UNIX path too long` from pytest generated temp_path
+        with tempfile.TemporaryDirectory() as path:
+            if request.param == "path":
+                yield Path(path) / "socket"
+            else:
+                yield Path(f"\0{path}") / "socket"
 
     @pytest.fixture(params=[False, True], ids=["str", "path"])
     def socket_path_or_str(self, request: SubRequest, socket_path: Path) -> Path | str:
@@ -731,7 +776,15 @@ class TestUNIXStream:
             assert (
                 stream.extra(SocketAttribute.local_address) == raw_socket.getsockname()
             )
-            assert stream.extra(SocketAttribute.remote_address) == str(socket_path)
+            remote_addr = stream.extra(SocketAttribute.remote_address)
+            if isinstance(remote_addr, str):
+                assert stream.extra(SocketAttribute.remote_address) == str(socket_path)
+            else:
+                assert isinstance(remote_addr, bytes)
+                assert stream.extra(SocketAttribute.remote_address) == bytes(
+                    socket_path
+                )
+
             pytest.raises(
                 TypedAttributeLookupError, stream.extra, SocketAttribute.local_port
             )
@@ -998,17 +1051,50 @@ class TestUNIXStream:
             await stream.send(b"foo")
 
     async def test_cannot_connect(self, socket_path: Path) -> None:
-        with pytest.raises(FileNotFoundError):
-            await connect_unix(socket_path)
+        if str(socket_path).startswith("\0"):
+            with pytest.raises(ConnectionRefusedError):
+                await connect_unix(socket_path)
+        else:
+            with pytest.raises(FileNotFoundError):
+                await connect_unix(socket_path)
+
+    async def test_connecting_using_bytes(
+        self, server_sock: socket.socket, socket_path: Path
+    ) -> None:
+        async with await connect_unix(str(socket_path).encode()):
+            pass
+
+    @pytest.mark.skipif(
+        platform.system() == "Darwin", reason="macOS requires valid UTF-8 paths"
+    )
+    async def test_connecting_with_non_utf8(self, socket_path: Path) -> None:
+        actual_path = str(socket_path).encode() + b"\xf0"
+        with socket.socket(socket.AF_UNIX) as server:
+            server.bind(actual_path)
+            server.listen(1)
+
+            async with await connect_unix(actual_path):
+                pass
 
 
 @pytest.mark.skipif(
     sys.platform == "win32", reason="UNIX sockets are not available on Windows"
 )
 class TestUNIXListener:
-    @pytest.fixture
-    def socket_path(self, tmp_path_factory: TempPathFactory) -> Path:
-        return tmp_path_factory.mktemp("unix").joinpath("socket")
+    @pytest.fixture(
+        params=[
+            "path",
+            pytest.param("abstract", marks=[skip_unix_abstract_mark]),
+        ]
+    )
+    def socket_path(self, request: SubRequest) -> Generator[Path, None, None]:
+        # Use stdlib tempdir generation
+        # Fixes `OSError: AF_UNIX path too long` from pytest generated temp_path
+        with tempfile.TemporaryDirectory() as path:
+            if request.param == "path":
+                yield Path(path) / "socket"
+            else:
+                yield Path(f"\0{path}") / "socket"
 
     @pytest.fixture(params=[False, True], ids=["str", "path"])
     def socket_path_or_str(self, request: SubRequest, socket_path: Path) -> Path | str:
@@ -1087,7 +1173,7 @@ class TestUNIXListener:
                         try:
                             message = client.recv(10)
                         except BlockingIOError:
-                            await sleep(0)
+                            await checkpoint()
                         else:
                             assert message == b"Hello\n"
                             break
@@ -1100,6 +1186,18 @@ class TestUNIXListener:
             async with await create_unix_listener(socket_path):
                 pass
 
+    async def test_listening_bytes_path(self, socket_path: Path) -> None:
+        async with await create_unix_listener(str(socket_path).encode()):
+            pass
+
+    @pytest.mark.skipif(
+        platform.system() == "Darwin", reason="macOS requires valid UTF-8 paths"
+    )
+    async def test_listening_invalid_ascii(self, socket_path: Path) -> None:
+        real_path = str(socket_path).encode() + b"\xf0"
+        async with await create_unix_listener(real_path):
+            pass
+
 
 async def test_multi_listener(tmp_path_factory: TempPathFactory) -> None:
     async def handle(stream: SocketStream) -> None:
@@ -1109,37 +1207,40 @@ async def test_multi_listener(tmp_path_factory: TempPathFactory) -> None:
 
     client_addresses: list[str | IPSockAddrType] = []
     listeners: list[Listener] = [await create_tcp_listener(local_host="localhost")]
-    if sys.platform != "win32":
-        socket_path = tmp_path_factory.mktemp("unix").joinpath("socket")
-        listeners.append(await create_unix_listener(socket_path))
+    with tempfile.TemporaryDirectory() as path:
+        if sys.platform != "win32":
+            listeners.append(await create_unix_listener(Path(path) / "socket"))
 
-    expected_addresses: list[str | IPSockAddrType] = []
-    async with MultiListener(listeners) as multi_listener:
-        async with create_task_group() as tg:
-            tg.start_soon(multi_listener.serve, handle)
-            for listener in multi_listener.listeners:
-                event = Event()
-                local_address = listener.extra(SocketAttribute.local_address)
-                if (
-                    sys.platform != "win32"
-                    and listener.extra(SocketAttribute.family)
-                    == socket.AddressFamily.AF_UNIX
-                ):
-                    assert isinstance(local_address, str)
-                    stream: SocketStream = await connect_unix(local_address)
-                else:
-                    assert isinstance(local_address, tuple)
-                    stream = await connect_tcp(*local_address)
+        expected_addresses: list[str | IPSockAddrType] = []
+        async with MultiListener(listeners) as multi_listener:
+            async with create_task_group() as tg:
+                tg.start_soon(multi_listener.serve, handle)
+                for listener in multi_listener.listeners:
+                    event = Event()
+                    local_address = listener.extra(SocketAttribute.local_address)
+                    if (
+                        sys.platform != "win32"
+                        and listener.extra(SocketAttribute.family)
+                        == socket.AddressFamily.AF_UNIX
+                    ):
+                        assert isinstance(local_address, str)
+                        stream: SocketStream = await connect_unix(local_address)
+                    else:
+                        assert isinstance(local_address, tuple)
+                        stream = await connect_tcp(*local_address)
 
-                expected_addresses.append(stream.extra(SocketAttribute.local_address))
-                await event.wait()
-                await stream.aclose()
+                    expected_addresses.append(
+                        stream.extra(SocketAttribute.local_address)
+                    )
+                    await event.wait()
+                    await stream.aclose()
 
-            tg.cancel_scope.cancel()
+                tg.cancel_scope.cancel()
 
-    assert client_addresses == expected_addresses
+        assert client_addresses == expected_addresses
 
 
+@pytest.mark.network
 @pytest.mark.usefixtures("check_asyncio_bug")
 class TestUDPSocket:
     async def test_extra_attributes(self, family: AnyIPAddressFamily) -> None:
@@ -1183,7 +1284,9 @@ class TestUDPSocket:
         async with await create_udp_socket(
             family=family, local_host="localhost"
         ) as server:
-            host, port = server.extra(SocketAttribute.local_address)  # type: ignore[misc]
+            host, port = server.extra(  # type: ignore[misc]
+                SocketAttribute.local_address
+            )
             async with await create_udp_socket(
                 family=family, local_host="localhost"
             ) as client:
@@ -1263,6 +1366,7 @@ class TestUDPSocket:
             assert local_address[1] > 0
 
 
+@pytest.mark.network
 @pytest.mark.usefixtures("check_asyncio_bug")
 class TestConnectedUDPSocket:
     async def test_extra_attributes(self, family: AnyIPAddressFamily) -> None:
@@ -1289,7 +1393,9 @@ class TestConnectedUDPSocket:
             async with await create_connected_udp_socket(
                 host, port, local_host="localhost", family=family
             ) as udp2:
-                host, port = udp2.extra(SocketAttribute.local_address)  # type: ignore[misc]
+                host, port = udp2.extra(
+                    SocketAttribute.local_address  # type: ignore[misc]
+                )
                 await udp2.send(b"blah")
                 request = await udp1.receive()
                 assert request == (b"blah", (host, port))
@@ -1308,7 +1414,9 @@ class TestConnectedUDPSocket:
         ) as udp1:
             host, port = udp1.extra(SocketAttribute.local_address)  # type: ignore[misc]
             async with await create_connected_udp_socket(host, port) as udp2:
-                host, port = udp2.extra(SocketAttribute.local_address)  # type: ignore[misc]
+                host, port = udp2.extra(  # type: ignore[misc]
+                    SocketAttribute.local_address
+                )
                 async with create_task_group() as tg:
                     tg.start_soon(serve)
                     await udp1.sendto(b"FOOBAR", host, port)
@@ -1381,6 +1489,322 @@ class TestConnectedUDPSocket:
             await udp.send(b"foo")
 
 
+@pytest.mark.skipif(
+    sys.platform == "win32", reason="UNIX sockets are not available on Windows"
+)
+class TestUNIXDatagramSocket:
+    @pytest.fixture(
+        params=[
+            "path",
+            pytest.param("abstract", marks=[skip_unix_abstract_mark]),
+        ]
+    )
+    def socket_path(self, request: SubRequest) -> Generator[Path, None, None]:
+        # Use stdlib tempdir generation
+        # Fixes `OSError: AF_UNIX path too long` from pytest generated temp_path
+        with tempfile.TemporaryDirectory() as path:
+            if request.param == "path":
+                yield Path(path) / "socket"
+            else:
+                yield Path(f"\0{path}") / "socket"
+
+    @pytest.fixture(params=[False, True], ids=["str", "path"])
+    def socket_path_or_str(self, request: SubRequest, socket_path: Path) -> Path | str:
+        return socket_path if request.param else str(socket_path)
+
+    @pytest.fixture
+    def peer_socket_path(self) -> Generator[Path, None, None]:
+        # Use stdlib tempdir generation
+        # Fixes `OSError: AF_UNIX path too long` from pytest generated temp_path
+        with tempfile.TemporaryDirectory() as path:
+            yield Path(path) / "peer_socket"
+
+    async def test_extra_attributes(self, socket_path: Path) -> None:
+        async with await create_unix_datagram_socket(local_path=socket_path) as unix_dg:
+            raw_socket = unix_dg.extra(SocketAttribute.raw_socket)
+            assert raw_socket.gettimeout() == 0
+            assert unix_dg.extra(SocketAttribute.family) == socket.AF_UNIX
+            assert (
+                unix_dg.extra(SocketAttribute.local_address) == raw_socket.getsockname()
+            )
+            pytest.raises(
+                TypedAttributeLookupError, unix_dg.extra, SocketAttribute.local_port
+            )
+            pytest.raises(
+                TypedAttributeLookupError, unix_dg.extra, SocketAttribute.remote_address
+            )
+            pytest.raises(
+                TypedAttributeLookupError, unix_dg.extra, SocketAttribute.remote_port
+            )
+
+    async def test_send_receive(self, socket_path_or_str: Path | str) -> None:
+        async with await create_unix_datagram_socket(
+            local_path=socket_path_or_str,
+        ) as sock:
+            path = str(socket_path_or_str)
+
+            await sock.sendto(b"blah", path)
+            request, addr = await sock.receive()
+            assert request == b"blah"
+            if isinstance(addr, bytes):
+                assert addr == path.encode()
+            else:
+                assert addr == path
+
+            await sock.sendto(b"halb", path)
+            response, addr = await sock.receive()
+            assert response == b"halb"
+            if isinstance(addr, bytes):
+                assert addr == path.encode()
+            else:
+                assert addr == path
+
+    async def test_iterate(self, peer_socket_path: Path, socket_path: Path) -> None:
+        async def serve() -> None:
+            async for packet, addr in server:
+                await server.send((packet[::-1], addr))
+
+        async with await create_unix_datagram_socket(
+            local_path=peer_socket_path,
+        ) as server:
+            peer_path = str(peer_socket_path)
+            async with await create_unix_datagram_socket(
+                local_path=socket_path
+            ) as client:
+                async with create_task_group() as tg:
+                    tg.start_soon(serve)
+                    await client.sendto(b"FOOBAR", peer_path)
+                    assert await client.receive() == (b"RABOOF", peer_path)
+                    await client.sendto(b"123456", peer_path)
+                    assert await client.receive() == (b"654321", peer_path)
+                    tg.cancel_scope.cancel()
+
+    async def test_concurrent_receive(self) -> None:
+        async with await create_unix_datagram_socket() as unix_dg:
+            async with create_task_group() as tg:
+                tg.start_soon(unix_dg.receive)
+                await wait_all_tasks_blocked()
+                try:
+                    with pytest.raises(BusyResourceError) as exc:
+                        await unix_dg.receive()
+
+                    exc.match("already reading from")
+                finally:
+                    tg.cancel_scope.cancel()
+
+    async def test_close_during_receive(self) -> None:
+        async def close_when_blocked() -> None:
+            await wait_all_tasks_blocked()
+            await unix_dg.aclose()
+
+        async with await create_unix_datagram_socket() as unix_dg:
+            async with create_task_group() as tg:
+                tg.start_soon(close_when_blocked)
+                with pytest.raises(ClosedResourceError):
+                    await unix_dg.receive()
+
+    async def test_receive_after_close(self) -> None:
+        unix_dg = await create_unix_datagram_socket()
+        await unix_dg.aclose()
+        with pytest.raises(ClosedResourceError):
+            await unix_dg.receive()
+
+    async def test_send_after_close(self, socket_path: Path) -> None:
+        unix_dg = await create_unix_datagram_socket(local_path=socket_path)
+        path = str(socket_path)
+        await unix_dg.aclose()
+        with pytest.raises(ClosedResourceError):
+            await unix_dg.sendto(b"foo", path)
+
+    async def test_local_path_bytes(self, socket_path: Path) -> None:
+        async with await create_unix_datagram_socket(
+            local_path=str(socket_path).encode()
+        ):
+            pass
+
+    @pytest.mark.skipif(
+        platform.system() == "Darwin", reason="macOS requires valid UTF-8 paths"
+    )
+    async def test_local_path_invalid_ascii(self, socket_path: Path) -> None:
+        real_path = str(socket_path).encode() + b"\xf0"
+        async with await create_unix_datagram_socket(local_path=real_path):
+            pass
+
+
+@pytest.mark.skipif(
+    sys.platform == "win32", reason="UNIX sockets are not available on Windows"
+)
+class TestConnectedUNIXDatagramSocket:
+    @pytest.fixture(
+        params=[
+            "path",
+            pytest.param("abstract", marks=[skip_unix_abstract_mark]),
+        ]
+    )
+    def socket_path(self, request: SubRequest) -> Generator[Path, None, None]:
+        # Use stdlib tempdir generation
+        # Fixes `OSError: AF_UNIX path too long` from pytest generated temp_path
+        with tempfile.TemporaryDirectory() as path:
+            if request.param == "path":
+                yield Path(path) / "socket"
+            else:
+                yield Path(f"\0{path}") / "socket"
+
+    @pytest.fixture(params=[False, True], ids=["str", "path"])
+    def socket_path_or_str(self, request: SubRequest, socket_path: Path) -> Path | str:
+        return socket_path if request.param else str(socket_path)
+
+    @pytest.fixture(
+        params=[
+            pytest.param("path", id="path-peer"),
+            pytest.param(
+                "abstract", marks=[skip_unix_abstract_mark], id="abstract-peer"
+            ),
+        ]
+    )
+    def peer_socket_path(self) -> Generator[Path, None, None]:
+        # Use stdlib tempdir generation
+        # Fixes `OSError: AF_UNIX path too long` from pytest generated temp_path
+        with tempfile.TemporaryDirectory() as path:
+            yield Path(path) / "peer_socket"
+
+    @pytest.fixture(params=[False, True], ids=["peer_str", "peer_path"])
+    def peer_socket_path_or_str(
+        self, request: SubRequest, peer_socket_path: Path
+    ) -> Path | str:
+        return peer_socket_path if request.param else str(peer_socket_path)
+
+    @pytest.fixture
+    def peer_sock(self, peer_socket_path: Path) -> Iterable[socket.socket]:
+        sock = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)
+        sock.settimeout(1)
+        sock.bind(str(peer_socket_path))
+        yield sock
+        sock.close()
+
+    async def test_extra_attributes(
+        self,
+        socket_path: Path,
+        peer_socket_path: Path,
+        peer_sock: socket.socket,
+    ) -> None:
+        async with await create_connected_unix_datagram_socket(
+            remote_path=peer_socket_path,
+            local_path=socket_path,
+        ) as unix_dg:
+            raw_socket = unix_dg.extra(SocketAttribute.raw_socket)
+            assert raw_socket is not None
+            assert unix_dg.extra(SocketAttribute.family) == AddressFamily.AF_UNIX
+            assert os.fsencode(
+                cast(os.PathLike, unix_dg.extra(SocketAttribute.local_address))
+            ) == os.fsencode(socket_path)
+            assert os.fsencode(
+                cast(os.PathLike, unix_dg.extra(SocketAttribute.remote_address))
+            ) == os.fsencode(peer_socket_path)
+            pytest.raises(
+                TypedAttributeLookupError, unix_dg.extra, SocketAttribute.local_port
+            )
+            pytest.raises(
+                TypedAttributeLookupError, unix_dg.extra, SocketAttribute.remote_port
+            )
+
+    async def test_send_receive(
+        self,
+        socket_path_or_str: Path | str,
+        peer_socket_path_or_str: Path | str,
+    ) -> None:
+        async with await create_unix_datagram_socket(
+            local_path=peer_socket_path_or_str,
+        ) as unix_dg1:
+            async with await create_connected_unix_datagram_socket(
+                peer_socket_path_or_str,
+                local_path=socket_path_or_str,
+            ) as unix_dg2:
+                socket_path = os.fsdecode(socket_path_or_str)
+
+                await unix_dg2.send(b"blah")
+                data, remote_addr = await unix_dg1.receive()
+                assert (data, os.fsdecode(remote_addr)) == (b"blah", socket_path)
+
+                await unix_dg1.sendto(b"halb", socket_path)
+                response = await unix_dg2.receive()
+                assert response == b"halb"
+
+    async def test_iterate(
+        self,
+        socket_path: Path,
+        peer_socket_path: Path,
+    ) -> None:
+        async def serve() -> None:
+            async for packet in unix_dg2:
+                await unix_dg2.send(packet[::-1])
+
+        async with await create_unix_datagram_socket(
+            local_path=peer_socket_path,
+        ) as unix_dg1:
+            async with await create_connected_unix_datagram_socket(
+                peer_socket_path, local_path=socket_path
+            ) as unix_dg2:
+                path = os.fsdecode(socket_path)
+                async with create_task_group() as tg:
+                    tg.start_soon(serve)
+                    await unix_dg1.sendto(b"FOOBAR", path)
+                    data, addr = await unix_dg1.receive()
+                    assert (data, os.fsdecode(addr)) == (b"RABOOF", path)
+                    await unix_dg1.sendto(b"123456", path)
+                    data, addr = await unix_dg1.receive()
+                    assert (data, os.fsdecode(addr)) == (b"654321", path)
+                    tg.cancel_scope.cancel()
+
+    async def test_concurrent_receive(
+        self, peer_socket_path: Path, peer_sock: socket.socket
+    ) -> None:
+        async with await create_connected_unix_datagram_socket(
+            peer_socket_path
+        ) as unix_dg:
+            async with create_task_group() as tg:
+                tg.start_soon(unix_dg.receive)
+                await wait_all_tasks_blocked()
+                try:
+                    with pytest.raises(BusyResourceError) as exc:
+                        await unix_dg.receive()
+
+                    exc.match("already reading from")
+                finally:
+                    tg.cancel_scope.cancel()
+
+    async def test_close_during_receive(
+        self, peer_socket_path_or_str: Path | str, peer_sock: socket.socket
+    ) -> None:
+        async def close_when_blocked() -> None:
+            await wait_all_tasks_blocked()
+            await udp.aclose()
+
+        async with await create_connected_unix_datagram_socket(
+            peer_socket_path_or_str
+        ) as udp:
+            async with create_task_group() as tg:
+                tg.start_soon(close_when_blocked)
+                with pytest.raises(ClosedResourceError):
+                    await udp.receive()
+
+    async def test_receive_after_close(
+        self, peer_socket_path_or_str: Path | str, peer_sock: socket.socket
+    ) -> None:
+        udp = await create_connected_unix_datagram_socket(peer_socket_path_or_str)
+        await udp.aclose()
+        with pytest.raises(ClosedResourceError):
+            await udp.receive()
+
+    async def test_send_after_close(
+        self, peer_socket_path_or_str: Path | str, peer_sock: socket.socket
+    ) -> None:
+        udp = await create_connected_unix_datagram_socket(peer_socket_path_or_str)
+        await udp.aclose()
+        with pytest.raises(ClosedResourceError):
+            await udp.send(b"foo")
+
+
 @pytest.mark.network
 async def test_getaddrinfo() -> None:
     # IDNA 2003 gets this wrong
