@@ -22,6 +22,7 @@ from anyio import (
     get_current_task,
     move_on_after,
     sleep,
+    sleep_forever,
     wait_all_tasks_blocked,
 )
 from anyio.abc import TaskGroup, TaskStatus
@@ -123,6 +124,16 @@ async def test_start_called_twice() -> None:
         assert value is None
 
 
+async def test_no_called_started_twice() -> None:
+    async def taskfunc(*, task_status: TaskStatus) -> None:
+        task_status.started()
+
+    async with create_task_group() as tg:
+        coro = tg.start(taskfunc)
+        tg.cancel_scope.cancel()
+        await coro
+
+
 async def test_start_with_value() -> None:
     async def taskfunc(*, task_status: TaskStatus) -> None:
         task_status.started("foo")
@@ -185,9 +196,6 @@ async def test_start_cancelled() -> None:
     assert not finished
 
 
-@pytest.mark.xfail(
-    sys.version_info < (3, 9), reason="Requires a way to detect cancellation source"
-)
 @pytest.mark.parametrize("anyio_backend", ["asyncio"])
 async def test_start_native_host_cancelled() -> None:
     started = finished = False
@@ -313,7 +321,7 @@ async def test_level_cancellation() -> None:
         nonlocal marker
         marker = 1
         # At this point the task has been cancelled so sleep() will raise an exception
-        await sleep(0)
+        await checkpoint()
         # Execution should never get this far
         marker = 2
 
@@ -374,7 +382,7 @@ async def test_cancel_scope_in_another_task() -> None:
     async with create_task_group() as tg:
         tg.start_soon(child)
         while local_scope is None:
-            await sleep(0)
+            await checkpoint()
 
         local_scope.cancel()
 
@@ -390,7 +398,7 @@ async def test_cancel_propagation() -> None:
 
     async with create_task_group() as tg:
         tg.start_soon(g)
-        await sleep(0)
+        await checkpoint()
         tg.cancel_scope.cancel()
 
 
@@ -447,6 +455,19 @@ async def test_cancel_before_entering_scope() -> None:
         pytest.fail("execution should not reach this point")
 
 
+@pytest.mark.xfail(
+    sys.version_info < (3, 11), reason="Requires asyncio.Task.cancelling()"
+)
+@pytest.mark.parametrize("anyio_backend", ["asyncio"])
+async def test_cancel_counter_nested_scopes() -> None:
+    with CancelScope() as root_scope:
+        with CancelScope():
+            root_scope.cancel()
+            await sleep(0.5)
+
+    assert not cast(asyncio.Task, asyncio.current_task()).cancelling()
+
+
 async def test_exception_group_children() -> None:
     with pytest.raises(BaseExceptionGroup) as exc:
         async with create_task_group() as tg:
@@ -471,14 +492,14 @@ async def test_exception_group_host() -> None:
 async def test_escaping_cancelled_exception() -> None:
     async with create_task_group() as tg:
         tg.cancel_scope.cancel()
-        await sleep(0)
+        await checkpoint()
 
 
 async def test_cancel_scope_cleared() -> None:
     with move_on_after(0.1):
         await sleep(1)
 
-    await sleep(0)
+    await checkpoint()
 
 
 @pytest.mark.parametrize("delay", [0, 0.1], ids=["instant", "delayed"])
@@ -533,8 +554,8 @@ async def test_fail_after_cancelled_before_deadline() -> None:
 async def test_fail_after_scope_cancelled_before_timeout() -> None:
     with fail_after(0.1) as scope:
         scope.cancel()
-        time.sleep(0.11)
-        await sleep(0)
+        time.sleep(0.11)  # noqa: ASYNC251
+        await checkpoint()
 
 
 @pytest.mark.parametrize("delay", [0, 0.1], ids=["instant", "delayed"])
@@ -607,8 +628,11 @@ async def test_cancel_from_shielded_scope() -> None:
             assert inner_scope.shield
             tg.cancel_scope.cancel()
             assert current_effective_deadline() == math.inf
+            assert not get_current_task().has_pending_cancellation()
+            await checkpoint()
 
         assert current_effective_deadline() == -math.inf
+        assert get_current_task().has_pending_cancellation()
 
         with pytest.raises(get_cancelled_exc_class()):
             await sleep(0.01)
@@ -622,13 +646,14 @@ async def test_cancel_shielded_scope() -> None:
         assert cancel_scope.shield
         cancel_scope.cancel()
         assert current_effective_deadline() == -math.inf
+        assert get_current_task().has_pending_cancellation()
 
         with pytest.raises(get_cancelled_exc_class()):
-            await sleep(0)
+            await checkpoint()
 
 
 async def test_cancelled_not_caught() -> None:
-    with CancelScope() as scope:
+    with CancelScope() as scope:  # noqa:  ASYNC100
         scope.cancel()
 
     assert scope.cancel_called
@@ -645,12 +670,13 @@ async def test_cancel_host_asyncgen() -> None:
             with CancelScope(shield=True) as inner_scope:
                 assert inner_scope.shield
                 tg.cancel_scope.cancel()
+                await checkpoint()
 
             with pytest.raises(get_cancelled_exc_class()):
-                await sleep(0)
+                await checkpoint()
 
             with pytest.raises(get_cancelled_exc_class()):
-                await sleep(0)
+                await checkpoint()
 
             done = True
 
@@ -785,11 +811,12 @@ async def test_shielded_deadline() -> None:
         with CancelScope(shield=True):
             with move_on_after(1000):
                 assert current_effective_deadline() - current_time() > 900
+                await checkpoint()
 
 
 async def test_deadline_reached_on_start() -> None:
     with move_on_after(0):
-        await sleep(0)
+        await checkpoint()
         pytest.fail("Execution should not reach this point")
 
 
@@ -857,6 +884,7 @@ async def test_triple_nested_shield_checkpoint_in_outer() -> None:
                 with CancelScope(shield=True):
                     scope1.cancel()
                     scope2.cancel()
+                    await checkpoint()
 
             await checkpoint()
             got_past_checkpoint = True
@@ -877,6 +905,7 @@ async def test_triple_nested_shield_checkpoint_in_middle() -> None:
             with CancelScope():
                 with CancelScope(shield=True):
                     scope1.cancel()
+                    await checkpoint()
 
                 await checkpoint()
                 got_past_checkpoint = True
@@ -1252,7 +1281,7 @@ class TestUncancel:
         task = cast(asyncio.Task, asyncio.current_task())
         with pytest.raises(asyncio.CancelledError), CancelScope():
             task.cancel()
-            await anyio.sleep(0)
+            await checkpoint()
 
         assert task.cancelling() == 1
         task.uncancel()
@@ -1261,7 +1290,7 @@ class TestUncancel:
         task = cast(asyncio.Task, asyncio.current_task())
         with CancelScope() as scope:
             scope.cancel()
-            await anyio.sleep(0)
+            await checkpoint()
 
         assert task.cancelling() == 0
 
@@ -1270,7 +1299,7 @@ class TestUncancel:
         with pytest.raises(asyncio.CancelledError), CancelScope() as scope:
             scope.cancel()
             task.cancel()
-            await anyio.sleep(0)
+            await checkpoint()
 
         assert task.cancelling() == 1
         task.uncancel()
@@ -1280,13 +1309,13 @@ class TestUncancel:
         assert task
         try:
             task.cancel()
-            await anyio.sleep(0)
+            await checkpoint()
         except asyncio.CancelledError:
             try:
                 with CancelScope() as scope:
                     scope.cancel()
                     try:
-                        await anyio.sleep(0)
+                        await checkpoint()
                     except asyncio.CancelledError:
                         raise asyncio.CancelledError
             except asyncio.CancelledError:
@@ -1336,6 +1365,24 @@ async def test_cancel_child_task_when_host_is_shielded() -> None:
                 await cancelled.wait()
 
 
+async def test_start_cancels_parent_scope() -> None:
+    """Regression test for #685 / #710."""
+    started: bool = False
+
+    async def in_task_group(task_status: TaskStatus[None]) -> None:
+        nonlocal started
+        started = True
+        await sleep_forever()
+
+    async with create_task_group() as tg:
+        with CancelScope() as inner_scope:
+            inner_scope.cancel()
+            await tg.start(in_task_group)
+
+    assert started
+    assert not tg.cancel_scope.cancel_called
+
+
 class TestTaskStatusTyping:
     """
     These tests do not do anything at run time, but since the test suite is also checked
